@@ -1,11 +1,17 @@
 import sys
+import warnings
 from threading import Thread
 
-from cartopy import crs
 import time
 import holoviews as hv
 import panel as pn
-import cartopy.crs as ccrs
+try:
+    from cartopy import crs
+    import cartopy.crs as ccrs
+    projection = ccrs.PlateCarree()
+except:
+    warnings.warn("Unable to import cartopy. For better plots install cartopy!")
+    projection = None
 
 import pandas as pd
 import hvplot.pandas
@@ -13,7 +19,6 @@ import hvplot.xarray
 import hvplot
 from bokeh.plotting import show
 import xarray as xr
-import warnings
 import copy
 import numpy as np
 
@@ -64,10 +69,6 @@ def change_coords(dx):
     lat_df.columns = ["value", "number"]
 
     if lat_df.number[0] / lat_df.number[1] < 10:
-        #    switch_coords = True
-
-        ## if there are repeated zeros
-        # if len([x for x in ds[lat_name].values[0] if x == 0]) <= 1:
         return ds
 
     if sum(ds[lon_name].values[0] == ds[lon_name].values[1]) == len(ds[lon_name][0]):
@@ -173,9 +174,16 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
     else:
         xr_file = False
 
+    nc_vars = None
     if xr_file is False:
         try:
-            ds = xr.open_dataset(x)
+            try:
+                import nctoolkit as nc
+                ds = nc.open_data(x)
+                nc_vars = ds.variables
+                ds = ds.to_xarray()
+            except:
+                ds = xr.open_dataset(x)
         except:
             ds = xr.open_dataset(x, decode_times=False)
             warnings.warn("Warning: xarray could not decode times!")
@@ -217,6 +225,32 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
             if "pole" in ds[lon_name].long_name:
                 coastline = False
 
+    if True:
+        dims = list(ds.dims)
+        coords = ds.coords
+        coords_list = list(coords)
+        selected = []
+        for x in coords_list:
+            selected += list(coords[x].dims)
+        selected = list(set(selected))
+        add_these = [x for x in dims if x not in selected]
+
+        if len(add_these) > 0:
+            for x in add_these:
+                if x not in coords_list and "vert" not in x:
+                    ds[x] = ds[x]
+            df_dims = get_dims(ds)
+            lon_name = df_dims.longitude[0]
+            lat_name = df_dims.latitude[0]
+
+            if lon_name is not None:
+                if "long_name" in ds[lon_name].attrs:
+                    if "rotate" in ds[lon_name].long_name:
+                        coastline = False
+
+                    if "pole" in ds[lon_name].long_name:
+                        coastline = False
+
     if len([x for x in ds.coords if "lon" in x]) > len(
         [x for x in ds.dims if "lon" in x]
     ):
@@ -417,7 +451,13 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
 
     if len([x for x in coord_df.length if x > 1]) == 1 and spatial_map is False:
 
-        df = ds.to_dataframe().reset_index()
+        df = ds.to_dataframe()
+        if nc_vars is not None:
+            cols = [x for x in list(df.columns) if x in nc_vars]
+            df = df.loc[:,cols]
+
+        df = df.reset_index().drop_duplicates().reset_index()
+
         if type(vars) is str:
             vars = [vars]
 
@@ -601,6 +641,7 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                 time_in = True
                 possible += 1
 
+
         if time_name in coord_list and time_in and non_map:
 
             if coord_df.query("coord == @time_name").length.values > 1:
@@ -740,7 +781,12 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
 
             if coastline:
                 coastline = get_coastline(ds, lon_name, lat_name)
-                projection = ccrs.PlateCarree()
+                try:
+                    projection = ccrs.PlateCarree()
+                except:
+                    projection = None
+                    coastline = False
+                    
             else:
                 projection = None
 
@@ -760,7 +806,11 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
 
             if coastline:
                 coastline = get_coastline(ds, lon_name, lat_name)
-                projection = ccrs.PlateCarree()
+                try:
+                    projection = ccrs.PlateCarree()
+                except:
+                    projection = None
+                    coastline = False
             else:
                 projection = None
 
@@ -809,7 +859,11 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
             if quadmesh:
                 if coastline:
                     coastline = get_coastline(ds, lon_name, lat_name)
-                    projection = ccrs.PlateCarree()
+                    try:
+                        projection = ccrs.PlateCarree()
+                    except:
+                        projection = None
+                        coastline = False
                 else:
                     projection = None
 
@@ -839,11 +893,14 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
                         responsive=(in_notebook() is False),
                         **kwargs
                     )
-                # intplot = pn.Row(pn.WidgetBox(w1), intplot)
             else:
                 if coastline:
                     coastline = get_coastline(ds, lon_name, lat_name)
-                    projection = ccrs.PlateCarree()
+                    try:
+                        projection = ccrs.PlateCarree()
+                    except:
+                        projection = None
+                        coastline = False
                 else:
                     projection = None
 
@@ -892,9 +949,15 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
             if quadmesh:
                 if coastline:
                     coastline = get_coastline(ds, lon_name, lat_name)
-                    projection = ccrs.PlateCarree()
+                    try:
+                        projection = ccrs.PlateCarree()
+                    except:
+                        projection = None
+                        coastline = False
                 else:
                     projection = None
+
+
                 if autoscale:
                     intplot = ds.hvplot.quadmesh(
                         lon_name,
@@ -926,9 +989,14 @@ def view(x, vars=None, autoscale=True,out = None, **kwargs):
 
                 if coastline:
                     coastline = get_coastline(ds, lon_name, lat_name)
-                    projection = ccrs.PlateCarree()
+                    try:
+                        projection = ccrs.PlateCarree()
+                    except:
+                        projection = None
+                        coastline = False
                 else:
                     projection = None
+                projection = None
 
                 if autoscale:
                     intplot = ds.hvplot.image(
